// Run one test with -g + keyword:
//mocha -g syntax

//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------

const should  = require('should');
const sinon   = require('sinon');
const startCluster = require('./http-cluster.js')
const sendScriptJob = require('./http-client.js')
const debug = require('debug')('mocha');

//-----------------------------------------------------------------------------
// Tests
//-----------------------------------------------------------------------------

describe('Cluster', function(){

  it('client returns sandbox result', function(done) {
    startCluster( function(){
      debug('startCluster callback');

      var scriptObj = {
        script : 'wrapper=function(){ return 1+8 }; RESULT = wrapper()',
        context: {}
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        res.result.should.eql( 9 );
        done()
      })
    });
  });

  it('should timeout on infinite loops', function(done) {
    this.timeout(5500);


    startCluster( function(){

      var scriptObj = {
        script: 'while ( true ) {}',
        input: {}
       }
      sendScriptJob( scriptObj, function(err, res ){
        debug('err', err);
        err.statusCode.should.eql( 500 );
        done()
      })
    })
  });

  it('end-to-end: default context is inserted', function(done) {
    startCluster( function(){
      debug('startCluster callback');

      var scriptObj = {
        script : 'wrapper=function(){ console.log(INPUT); return INPUT.a + 8 }; RESULT = wrapper()',
        context: { INPUT : { a: 1 } }
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        // var parsed = JSON.parse(res)
        // assert.equal(  , "COMPLETE")
        res.result.should.eql( 9 );
        done()
      })
      // sendScriptJob()
    });
  });

  it('end-to-end: import npm libraries', function(done) {
    startCluster( function(){
      debug('startCluster callback');

      var myCode = `
        t.timeout(100, function(){
          done( { a: 1})
        })
      `

      var scriptObj = {
        script : myCode,
        context: { },
        libs : [{ namespace: 't', npm: 'timeout'}]
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        // var parsed = JSON.parse(res)
        // assert.equal(  , "COMPLETE")
        res.result.a.should.eql( 1 );
        done()
      })
      // sendScriptJob()
    });
  });

  it('end-to-end: external functions', function(done) {
    this.timeout(5000)
    startCluster( function(){
      debug('startCluster callback');

      var externalFunction = function( ) {
        return { a: 1 }
      }

      var myCode = `
      console.log( externalFunction() )
      RESULT =  externalFunction()
      `

      var scriptObj = {
        script : myCode,
        context: { externalFunction : externalFunction },
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        res.result.a.should.eql( 1 );
        done()
      });
    });
  });

  it('end-to-end: external request function', function(done) {
    startCluster( function(){
      debug('startCluster callback');

      // const request = require("request")

      var httpRequestFunction = function( requestObj ) {
        const request = require("request")

        return new Promise(function (resolve, reject) {
          request( requestObj , function (err, IncomingMessage, body) {
            if (err) {
              // console.log('httpRequest err', err);
              return reject(err);
            }
            resolve(body);
          });
        });
      }

      var myCode = `
      httpRequestFunction({
        url: 'http://mockbin.org/bin/40bd780c-a6ae-456c-8dbd-d61d9f47ba3b',
        json: true
      }).then( function ( htmlBody ) {
        console.log( 'htmlBody', htmlBody);
        done(htmlBody)
      }).catch( function(err) {
        console.log('err', err )
      })
      `

      var scriptObj = {
        script : myCode,
        context: { httpRequestFunction : httpRequestFunction },
        // libs : [{ namespace:'request', npm: 'request' }]
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        res.result.foo.should.eql( 'hello world' );
        done()
      })
    });
  });

  it('TODO: end-to-end: external request function (ideal)', function(done) {
    startCluster( function(){
      debug('startCluster callback');

      var httpRequestFunction = function( requestObj ) {
        //TODO: why can't we just include request in the whitelisted libraries. see test
        // request = require("request")

        return new Promise(function (resolve, reject) {
          request( requestObj , function (err, IncomingMessage, body) {
            if (err) {
              // console.log('httpRequest err', err);
              return reject(err);
            }
            resolve(body);
          });
        });
      }

      var myCode = `
      httpRequestFunction({
        url: 'http://mockbin.org/bin/40bd780c-a6ae-456c-8dbd-d61d9f47ba3b',
        json: true
      }).then( function ( htmlBody ) {
        console.log( 'htmlBody', htmlBody);
        done(htmlBody)
      }).catch( function(err) {
        console.log('err', err )
      })
      `

      var scriptObj = {
        script : myCode,
        context: {  httpRequestFunction : httpRequestFunction},
        //TODO: this should be enough
        libs : [{ namespace:'request', npm: 'request' }]
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        res.result.foo.should.eql( 'hello world' );
        done()
      })
    });
  });

  it('end-to-end: handle errors correctly', function(done) {
    startCluster( function(){
      debug('startCluster callback');

      var scriptObj = {
        script : 'wrapper=function(){ \
          \nreturn request({ url: "http://mockbin.org/bin/17889536-56e3-498d-9caa-be279a7e5a8d/" }).then(function( body ){ RESULT = body; done()})\
          \n }; RESULT = wrapper()',
        context: { INPUT : { a: 1 } }
      }

      sendScriptJob( scriptObj, function(err, res ){
        debug('res', res);
        debug('err', err);

        err.status.should.eql( 'ERROR' );
        err.result.should.eql( 'ReferenceError : request is not defined');

        done()
      });
    });
  });
});
